/* Copyright (C) 1994 Queen Mary and Westfield College, University of London

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   A copy of the GNU General Public License may be found in
   doc/COPYING in the source tree of this programm.

   This file was written by: Franz Gerschwiler.(goblin@dcs.qmw.ac.uk)

 ****

   modified by: p.hlstede	(peter@calvados.chemie.uni-bremen.de)

   - the code was reorganized and ansified
   - some callbacks were merged into one
   - word wrap and autoindent can now be toggeled
   - the horizontal scrollbar can be toggeled according to word wrap

							06.09.1997

   - the actual cursor position is now saved and restored, when word
     wrap is toggeled

							19.10.1997

 */

#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <ctype.h>

#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/ToggleB.h>

#include "sys_consts.h"
#include "globals.h"
#include "callbacks.h"
#include "widgets.h"
#include "popUps.h"
#include "fileEvents.h"
#include "optionEvents.h"
#include "undo.h"
#include "utils.h"
#include "dstring.h"

/* Paranoia macros (who knows how boolean is defined...) */
#define INT2BOOLEAN(i) ((i) ? True : False)
#define BOOLEAN2INT(b) ((b) ? 1 : 0)

/***  GLOBAL VARIABLES  ***/
int       AutoSaveTime;
Boolean   AutoSave;
Boolean   AutoBackup;
Boolean   AutoIndent;
Boolean   WordWrap;
Boolean   FileBox;
char     *DefaultPrinter;
char     *defaultPrinter = NULL;

ExtMode    *extmode = NULL;
static int  extcnt  = 0;
int         extnum  = 0;

/***  Copies for temporary storage  ***/
static int      autoSaveTime;
static Boolean  autoSave;
static Boolean  autoBackup;
static Boolean  autoIndent;
static Boolean  wordWrap;
static Boolean  fileBox;


/***  Local prototypes  ***/
static void show_option_values(void);
static void SetExtension(char *line);


#ifdef TOGGLE_SCROLLBAR
void  SetAutoIndentAndWordWrap(void)
{
   /* get the actual settings to avoid unnecessary settings of wordwrap */
   Boolean  actual_WordWrap;
   XtVaGetValues(text_canvas, XmNwordWrap, &actual_WordWrap, NULL);

   /* To toggle the horizontal scrollbar with the word wrap */
   /* we must destroy and recreate the textwidget.          */
   if (actual_WordWrap != WordWrap)
   {
      char *textcontents = (char *)XmTextGetString(text_canvas);
      XmTextPosition pos = XmTextGetInsertionPosition(text_canvas);
      XtDestroyWidget(text_canvas);
      create_text_canvas();
      if (textcontents != NULL)
      {
         undo_redo_action = True;	/* suppress an undo action for the text insertion */
         XmTextSetString(text_canvas, textcontents);
         XtFree(textcontents);
         
         XmTextSetInsertionPosition(text_canvas, pos);
         XmTextShowPosition(text_canvas, pos);
      }
      set_cursor(text_canvas, TextCursor);
   }

   if (AutoIndent)
      XtOverrideTranslations(text_canvas,
	    XtParseTranslationTable("<Key>Return: newline-and-indent()\n"));
   else
      XtOverrideTranslations(text_canvas,
            XtParseTranslationTable("<Key>Return: newline()\n"));
}
#else
void  SetAutoIndentAndWordWrap(void)
{
   if (AutoIndent)
      XtOverrideTranslations(text_canvas,
	    XtParseTranslationTable("<Key>Return: newline-and-indent()\n"));
   else
      XtOverrideTranslations(text_canvas,
	    XtParseTranslationTable("<Key>Return: newline()\n"));

   XtVaSetValues(text_canvas, XmNwordWrap, WordWrap, NULL);
}
#endif

void SetAutoSave(void)
{
   if (AutoSavePossible && AutoSave)
      XtAppAddTimeOut(app_context, AutoSaveTime*1000, AutoSaveFunc, NULL);

}


/* Now set the values in the setup popUp */
static void show_option_values(void)
{
   char  item[20];

   XmTextSetString(printerText, DefaultPrinter);
   sprintf(item, "%d", AutoSaveTime);
   XmTextSetString(autosavetimeText, item);

   XtVaSetValues(autosaveTB, XmNset, AutoSave,   NULL);
   XtVaSetValues(backupTB,   XmNset, AutoBackup, NULL);
   XtVaSetValues(autoindTB,  XmNset, AutoIndent, NULL);
   XtVaSetValues(wordwrapTB, XmNset, WordWrap,   NULL);
   XtVaSetValues(fileboxTB,  XmNset, FileBox,    NULL);
}


void  gui_setUpCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   /* Take a copy of the current values before allowing changes */
   autoSaveTime = AutoSaveTime;
   autoSave     = AutoSave;
   autoBackup   = AutoBackup;
   autoIndent   = AutoIndent;
   wordWrap     = WordWrap;
   fileBox      = FileBox;

   if (defaultPrinter)
      XtFree(defaultPrinter);
   defaultPrinter = Xstrdup(DefaultPrinter);

   if (configPopUp == (Widget) 0)
      gui_createConfigPopUp(toplevel);

   show_option_values();

   XtManageChild(configPopUp);
}


void  gui_saveConfigCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   FILE    *fpointer;
   char    *home;
   DString  filename;
   int      i;

   home = getenv("HOME");
   if (!home)
      home = ".";
   DStringCompose(&filename, home, "/.gloeConfig", NULL);
   fpointer = fopen(filename.str, "w");
   if (fpointer != NULL)
   {
      fprintf(fpointer, "AutoSaveTime %d\n", AutoSaveTime);
      fprintf(fpointer, "AutoSave     %d\n", BOOLEAN2INT(AutoSave));
      fprintf(fpointer, "Backup       %d\n", BOOLEAN2INT(AutoBackup));
      fprintf(fpointer, "AutoIndent   %d\n", BOOLEAN2INT(AutoIndent));
      fprintf(fpointer, "WordWrap     %d\n", BOOLEAN2INT(WordWrap));
      fprintf(fpointer, "FileBox      %d\n", BOOLEAN2INT(FileBox));
      fprintf(fpointer, "\n");
      fprintf(fpointer, "Printer      %s\n", DefaultPrinter);
      fprintf(fpointer, "\n");
      fprintf(fpointer, "# Extension WordWrap AutoIndent\n");
      for (i=0; i<extnum; i++)
         fprintf(fpointer, "*.%s\t\t%d\t%d\n", extmode[i].ext, extmode[i].ww, extmode[i].ai);
      fclose(fpointer);
   }
   DStringFree(&filename);
   return;
}

/* Fallback default values for setup */
void  setDefaults(void)
{
   AutoSaveTime = AUTO_SAVE_TIME;
   AutoSave     = True;
   AutoBackup   = True;
   AutoIndent   = True;
   WordWrap     = False;
   FileBox      = True;
   strcpy(DefaultPrinter, "No printer identified");
}


static void SetExtension(char *line)
{
   int  i;

   if (extnum >= extcnt)
   {
      extcnt += 10;
      extmode = (ExtMode*)XtRealloc((char*)extmode, extcnt * sizeof(ExtMode));
   }
   for (i=0; (i<15) && !isspace(line[i]); i++)
      extmode[extnum].ext[i] = line[i];
   extmode[extnum].ext[i] = '\0';
   line += i;

   extmode[extnum].ww = INT2BOOLEAN(strtol(line, &line, 10));
   extmode[extnum].ai = INT2BOOLEAN(strtol(line, &line, 10));

   extnum++;
}

/* Attempt to load user defined values for setup */
Boolean  loadConfig(void)
{
   FILE    *fpointer;
   char    *home;
   DString  filename;
   Boolean  result;

   home = getenv("HOME");
   if (!home) home = ".";
   DStringCompose(&filename, home, "/.gloeConfig", NULL);

   fpointer = fopen(filename.str, "rt");
   if (fpointer != NULL)
   {
      char  line[256];

      while(fgets(line, sizeof(line), fpointer))
      {
         char *p = line;
         while (*p && isspace(*p))
            ++p;
         if (!strncasecmp(p, "autosavetime", 12))
            AutoSaveTime = atoi(p+12);
         else if (!strncasecmp(p, "autosave", 8))
            AutoSave = INT2BOOLEAN(atoi(p+8));
         else if (!strncasecmp(p, "backup", 6))
            AutoBackup = INT2BOOLEAN(atoi(p+6));
         else if (!strncasecmp(p, "autoindent", 10))
            AutoIndent = INT2BOOLEAN(atoi(p+10));
         else if (!strncasecmp(p, "wordwrap", 8))
            WordWrap = INT2BOOLEAN(atoi(p+8));
         else if (!strncasecmp(p, "filebox", 7))
            FileBox = INT2BOOLEAN(atoi(p+7));
         else if (!strncasecmp(p, "printer", 7))
         {
            p += 7;
            while (*p && isspace(*p))
               ++p;
            strncpy(DefaultPrinter, p, MAX_PRINTER_LEN);
            p = DefaultPrinter + strlen(DefaultPrinter) - 1;
            while (isspace(*p))
               *p-- = '\0';
         }
         else if (!strncmp(p, "*.", 2))
            SetExtension(p+2);
      }

      fclose(fpointer);

      if (AutoSaveTime < MIN_AUTO_SAVE_TIME)
	 AutoSaveTime = MIN_AUTO_SAVE_TIME;

      result = True;
   }
   else
      result = False;

   DStringFree(&filename);

   return (result);
}


/* Callbacks for the config popUp */
/**********************************/

/* Restore original values of the configuration */
void  gui_restoreConfigCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   setDefaults();
   loadConfig();
   show_option_values();
}

void  gui_cancelConfigCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   /* return setup to values before the user made changes */
   AutoSaveTime = autoSaveTime;
   AutoSave     = autoSave;
   AutoBackup   = autoBackup;
   AutoIndent   = autoIndent;
   WordWrap     = wordWrap;
   FileBox      = fileBox;
   strcpy(DefaultPrinter, defaultPrinter);

   XtUnmanageChild(configPopUp);
}

void  gui_okConfigCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   gui_autosavetimeTextCB(autosavetimeText, NULL, NULL);
   gui_printerTextCB(printerText, NULL, NULL);

   XtUnmanageChild(configPopUp);
   if ((WordWrap != wordWrap) || (AutoIndent != autoIndent))
      SetAutoIndentAndWordWrap();
   if (AutoSave != autoSave)
      SetAutoSave();
}

void  gui_autosavetimeTextCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   char     txt[20];
   char    *temp;

   /* Get new period between autosaves */
   temp = XmTextGetString(autosavetimeText);
   AutoSaveTime = atoi(temp);
   if (temp != NULL)
      XtFree(temp);
   if (AutoSaveTime < MIN_AUTO_SAVE_TIME)
   {
      AutoSaveTime = MIN_AUTO_SAVE_TIME;
      sprintf(txt, "%d", AutoSaveTime);
      XmTextSetString(autosavetimeText, txt);
   }
}


void  gui_printerTextCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   char    *temp = XmTextGetString(printerText);
   strncpy(DefaultPrinter, temp, MAX_PRINTER_LEN);
   XtFree(temp);
}

void  gui_toggleItemCB(Widget w, XtPointer client_data, XtPointer call_data)
{
   int *p = (int*)client_data;
   *p = ((XmToggleButtonCallbackStruct *)call_data)->set;
}

